<?php
/**
 * Created by IntelliJ IDEA.
 * User: svip
 * Date: 19.07.13
 * Time: 17:09
 * To change this template use File | Settings | File Templates.
 */



/**
 * Implements hook_field_widget_settings_form().
 */
function select2widget_entityreference_widget_settings_form($field, $instance) {
  $widget = $instance['widget'];
  $settings = $widget['settings'] + field_info_widget_settings($widget['type']);

  $form = array();
  if ($widget['type'] == 'select2widgetajax') {
    $view_mode_options = array();

    $entity_info = entity_get_info();
    $entity_type = (isset($field['settings']['target_type'])) ? $field['settings']['target_type'] : NULL;

    if (isset($entity_info[$entity_type]['view modes'])) {
      foreach ($entity_info[$entity_type]['view modes'] as $key => $value) {
        $view_mode_options[$key] = $value['label'];
      }
    }

    $form['select2widgetAutocomplete'] = array(
      '#type' => 'fieldset',
      '#title' =>  t('Select2Widget settings'),
      '#tree' => TRUE,
    );

    $form['select2widgetAutocomplete']['view_mode'] = array(
      '#type' => 'select',
      '#title' => t('View mode'),
      '#default_value' => (isset($settings['select2widgetAutocomplete']['view_mode'])) ? $settings['select2widgetAutocomplete']['view_mode'] : reset($view_mode_options),
      '#options' => $view_mode_options,
      '#description' => t('Install !entity_view_mode_link module to create own view modes.',
        array('!entity_view_mode_link' => l(t('"Entity view modes"'), "http://drupal.org/project/entity_view_mode",
        array('attributes' => array('target' => '_blank'))))),
    );


    $form['select2widgetAutocomplete']['match_operator'] = array(
      '#type' => 'select',
      '#title' => t('Search matching'),
      '#default_value' => (isset($settings['select2widgetAutocomplete']['match_operator'])) ? $settings['select2widgetAutocomplete']['match_operator'] : "",
      '#options' => array(
        'STARTS_WITH' => t('Starts with'),
        'CONTAINS' => t('Contains'),
      ),
      '#description' => t('Select the method used to collect autocomplete suggestions. Note that <em>Contains</em> can cause performance issues on sites with thousands of nodes.'),
    );

    $form['select2widgetAutocomplete']['width'] = array(
      '#type' => 'textfield',
      '#size' => '5',
      '#title' => t('Widht of field (px)'),
      '#default_value' => (isset($settings['select2widgetAutocomplete']['width'])) ? $settings['select2widgetAutocomplete']['width'] : "",
      '#element_validate' => array('_element_validate_integer_positive'),
      '#required' => FALSE,
    );
  }

  return $form;
}


/**
 * Process callback: 'select2widget' element type.
 */
function select2widget_entity_process_callback($element, &$form_state, $form) {
  // Send Drupal.settings a reference to this form element.

  $cardinality = $form_state['field'][$element['#field_name']][$element['#language']]['field']['cardinality'];

  $data['select2widgetAutocomplete']['elements'][$element['#id']] = array(
    'id' => $element['#id'],
    'url' => $element['#autocomplete_path'],
    'cardinality' => $cardinality,
    'width' => $element['#widget_width'],
    'init' => $element['#init'],
  );

  // Attaching library, integration script, and settings array.
  $element['#attached']['library'][] = array('select2widget', 'select2');
  $element['#attached']['js'][] = array(
    'type' => 'setting',
    'data' => $data,
  );

  return $element;
}


/**
 * Return JSON based on given field, instance and string.
 *
 * This function can be used by other modules that wish to pass a mocked
 * definition of the field on instance.
 *
 * @param $field
 *   The field array defintion.
 * @param $instance
 *   The instance array defintion.
 * @param $entity_type
 *   The entity type.
 * @param $entity_id
 *   Optional; The entity ID the entity-reference field is attached to.
 *   Defaults to ''.
 * @param $string
 *   The label of the entity to query by.
 */
function select2widget_entityreference_get_matches($field, $instance, $entity_type, $entity_id = '', $string = '') {
  $matches = array();
  $options = array();
  $selected_target_ids = array();

  $entity = NULL;
  if ($entity_id !== 'NULL') {
    $entity = entity_load_single($entity_type, $entity_id);
    if (!$entity || !entity_access('view', $entity_type, $entity)) {
      return MENU_ACCESS_DENIED;
    }
  }

  $match_operator = $instance['widget']['settings']['select2widgetAutocomplete']['match_operator'];
  $entity_info = entity_get_info($field['settings']['target_type']);

  $items = field_get_items($entity_type, $entity, $field['field_name'], $entity->language);
  foreach ($items as $item) {
    $selected_target_ids[] = $item['target_id'];
  }

  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', $field['settings']['target_type']);

  if (!empty($field['settings']['handler_settings']['target_bundles'])) {
    $query->entityCondition('bundle', $field['settings']['handler_settings']['target_bundles'], 'IN');
  }

  if (!empty($selected_target_ids)) {
    $query->entityCondition('entity_id', $selected_target_ids, "NOT IN");
  }

  $query->propertyCondition($entity_info['entity keys']['label'], $string, $match_operator)
        ->range(0, 20);

  $results = $query->execute();

  if (!empty($results[$entity_type])) {
    $entities = entity_load($entity_type, array_keys($results[$entity_type]));

    foreach ($entities as $entity_id => $entity) {
      list(, , $bundle) = entity_extract_ids($entity_type, $entity);
      $options[$bundle][$entity_id] = check_plain(entity_label($entity_type, $entity));
    }
  }

  $matches = select2widget_render_modes($options, $instance['widget']['settings']['select2widgetAutocomplete']['view_mode'], $field['settings']['target_type']);

  drupal_json_output($matches);
}


/**
 * Render entities using display mode
 *
 * @param $entity_labels
 * @param string $view_mode
 * @return array
 */
function select2widget_render_modes($entity_labels, $view_mode = "full", $entity_type = "node") {
  $matches = array();

  foreach ($entity_labels as $values) {
    $ids = array_keys($values);
    $entities = entity_load_multiple_by_name($entity_type, $ids);
    foreach ($entities as $entity) {
      $entity_array = select2widget_render_entity($entity_type, $entity, $view_mode);
      $matches[] = array(
        'id' => $entity_array['id'],
        'title' => $entity_array['title'],
        'data' => '<div class="reference-select2widget">' . render($entity_array['data']) . '</div>',
      );
    }
  }

  return $matches;
}


/**
 * Render single entity
 *
 * @param $entity_type
 * @param $entity
 * @param $view_mode
 * @return array
 */
function select2widget_render_entity($entity_type, $entity, $view_mode = 'full') {
  $entity_array = array('id', 'title', 'data');
  switch ($entity_type) {
    case "node":
        $entity_array['id'] = $entity->nid;
        $entity_array['title'] = $entity->title . " - " . $entity->nid;
        $entity_array['data'] = node_view($entity, $view_mode);
      break;
    case "user":
        $entity_array['id'] = $entity->uid;
        $entity_array['title'] = $entity->name . " - " . $entity->uid;
        $entity_array['data'] = user_view($entity, $view_mode);
      break;
  }

  return $entity_array;
}

/**
 * Sets form value
 *
 * @param $element
 * @param $form_state
 * @param $form
 */
function select2widget_entity_validate_field(&$element, &$form_state, $form) {
  $value = array();
  $entity_labels = array();

  $settings = $element['#settings']['select2widgetAutocomplete'];
  $field = field_widget_field($element, $form_state);
  $instance = field_info_instance($element['#entity_type'], $element['#field_name'], $element['#bundle']);

  $target_entities = explode(',', $element['#value']);

  $handler = entityreference_get_selection_handler($field, $instance, $element['#entity_type'], $element['#entity']);

  foreach ($target_entities as &$target_entity) {
    if (is_numeric($target_entity)) {
      $value[] = array('target_id' => $target_entity);

      $entity = entity_load_single($element['#entity_type'], $target_entity);
      $label = $handler->getLabel($entity);
      $key = "$label ($target_entity)";
      // Labels containing commas or quotes must be wrapped in quotes.
      if (strpos($key, ',') !== FALSE || strpos($key, '"') !== FALSE) {
        $key = '"' . str_replace('"', '""', $key) . '"';
      }

      $entity_labels[$target_entity] = $key;
    }

    elseif (preg_match("/.+\((\d+)\)/", $target_entity, $matches)) {
      $entity_labels[$matches[1]] = $target_entity;
      $value[] = array('target_id' => $matches[1]);
    }

  }

  $element['#attached']['js'][0]['data']['select2widgetAutocomplete']['elements'][$element['#id']]['init'] = $entity_labels; //Update default values


  form_set_value($element, $value, $form_state);
}
